'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var accounts = require('../accounts.cjs');
var ListingReceiptGpaBuilder = require('../gpaBuilders/ListingReceiptGpaBuilder.cjs');
var Operation = require('../../../types/Operation.cjs');
var SdkError = require('../../../errors/SdkError.cjs');
var Listing = require('../models/Listing.cjs');

// Operation
// -----------------

const Key = 'FindListingsByPublicKeyOperation';
/**
 * Finds multiple Listings by specific criteria.
 *
 * ```ts
 * // Find listings by seller.
 * const listings = await metaplex
 *   .auctionHouse()
 *   .findListingsBy({ auctionHouse, type: 'seller', publicKey: seller };
 *
 * // Find listings by metadata.
 * const listings = await metaplex
 *   .auctionHouse()
 *   .findListingsBy({ auctionHouse, type: 'metadata', publicKey: metadata };
 *
 * // Find listings by mint.
 * const listings = await metaplex
 *   .auctionHouse()
 *   .findListingsBy({ auctionHouse, type: 'mint', publicKey: mint };
 * ```
 *
 * @group Operations
 * @category Constructors
 */

const findListingsByPublicKeyFieldOperation = Operation.useOperation(Key);
/**
 * @group Operations
 * @category Types
 */

/**
 * @group Operations
 * @category Handlers
 */
const findListingsByPublicKeyFieldOperationHandler = {
  handle: async (operation, metaplex, scope) => {
    const {
      commitment,
      programs
    } = scope;
    const {
      auctionHouse,
      type,
      publicKey
    } = operation.input;
    const auctionHouseProgram = metaplex.programs().getAuctionHouse();
    let listingQuery = new ListingReceiptGpaBuilder.ListingReceiptGpaBuilder(metaplex, auctionHouseProgram.address).mergeConfig({
      commitment
    }).whereAuctionHouse(auctionHouse.address);

    switch (type) {
      case 'seller':
        listingQuery = listingQuery.whereSeller(publicKey);
        break;

      case 'metadata':
        listingQuery = listingQuery.whereMetadata(publicKey);
        break;

      case 'mint':
        listingQuery = listingQuery.whereMetadata(metaplex.nfts().pdas().metadata({
          mint: publicKey,
          programs
        }));
        break;

      default:
        throw new SdkError.UnreachableCaseError(type);
    }

    scope.throwIfCanceled();
    return listingQuery.getAndMap(account => Listing.toLazyListing(accounts.toListingReceiptAccount(account), auctionHouse));
  }
};

exports.findListingsByPublicKeyFieldOperation = findListingsByPublicKeyFieldOperation;
exports.findListingsByPublicKeyFieldOperationHandler = findListingsByPublicKeyFieldOperationHandler;
//# sourceMappingURL=findListingsByPublicKeyField.cjs.map
