import { toListingReceiptAccount } from '../accounts.mjs';
import { ListingReceiptGpaBuilder } from '../gpaBuilders/ListingReceiptGpaBuilder.mjs';
import { useOperation } from '../../../types/Operation.mjs';
import { UnreachableCaseError } from '../../../errors/SdkError.mjs';
import { toLazyListing } from '../models/Listing.mjs';

// Operation
// -----------------

const Key = 'FindListingsByPublicKeyOperation';
/**
 * Finds multiple Listings by specific criteria.
 *
 * ```ts
 * // Find listings by seller.
 * const listings = await metaplex
 *   .auctionHouse()
 *   .findListingsBy({ auctionHouse, type: 'seller', publicKey: seller };
 *
 * // Find listings by metadata.
 * const listings = await metaplex
 *   .auctionHouse()
 *   .findListingsBy({ auctionHouse, type: 'metadata', publicKey: metadata };
 *
 * // Find listings by mint.
 * const listings = await metaplex
 *   .auctionHouse()
 *   .findListingsBy({ auctionHouse, type: 'mint', publicKey: mint };
 * ```
 *
 * @group Operations
 * @category Constructors
 */

const findListingsByPublicKeyFieldOperation = useOperation(Key);
/**
 * @group Operations
 * @category Types
 */

/**
 * @group Operations
 * @category Handlers
 */
const findListingsByPublicKeyFieldOperationHandler = {
  handle: async (operation, metaplex, scope) => {
    const {
      commitment,
      programs
    } = scope;
    const {
      auctionHouse,
      type,
      publicKey
    } = operation.input;
    const auctionHouseProgram = metaplex.programs().getAuctionHouse();
    let listingQuery = new ListingReceiptGpaBuilder(metaplex, auctionHouseProgram.address).mergeConfig({
      commitment
    }).whereAuctionHouse(auctionHouse.address);

    switch (type) {
      case 'seller':
        listingQuery = listingQuery.whereSeller(publicKey);
        break;

      case 'metadata':
        listingQuery = listingQuery.whereMetadata(publicKey);
        break;

      case 'mint':
        listingQuery = listingQuery.whereMetadata(metaplex.nfts().pdas().metadata({
          mint: publicKey,
          programs
        }));
        break;

      default:
        throw new UnreachableCaseError(type);
    }

    scope.throwIfCanceled();
    return listingQuery.getAndMap(account => toLazyListing(toListingReceiptAccount(account), auctionHouse));
  }
};

export { findListingsByPublicKeyFieldOperation, findListingsByPublicKeyFieldOperationHandler };
//# sourceMappingURL=findListingsByPublicKeyField.mjs.map
